@@ -1,31 +1,42 @@
 <?php
-//
-// +----------------------------------------------------------------------+
-// | Copyright (c) 1997-2005 Ulf Wendel, Pierre-Alain Joye                |
-// +----------------------------------------------------------------------+
-// | This source file is subject to the New BSD license, That is bundled  |
-// | with this package in the file LICENSE, and is available through      |
-// | the world-wide-web at                                                |
-// | http://www.opensource.org/licenses/bsd-license.php                   |
-// | If you did not receive a copy of the new BSDlicense and are unable   |
-// | to obtain it through the world-wide-web, please send a note to       |
-// | pajoye@php.net so we can mail you a copy immediately.                |
-// +----------------------------------------------------------------------+
-// | Author: Ulf Wendel <ulf.wendel@phpdoc.de>                            |
-// |         Pierre-Alain Joye <pajoye@php.net>                           |
-// +----------------------------------------------------------------------+
-//
-// $Id: IT.php,v 1.14 2005/11/01 10:14:19 pajoye Exp $
-//
+/**
+ * Integrated Template - IT
+ *
+ * PHP version 4
+ *
+ * Copyright (c) 1997-2007 Ulf Wendel, Pierre-Alain Joye,
+ *                         David Soria Parra
+ *
+ * This source file is subject to the New BSD license, That is bundled
+ * with this package in the file LICENSE, and is available through
+ * the world-wide-web at
+ * http://www.opensource.org/licenses/bsd-license.php
+ * If you did not receive a copy of the new BSDlicense and are unable
+ * to obtain it through the world-wide-web, please send a note to
+ * pajoye@php.net so we can mail you a copy immediately.
+ *
+ * Author: Ulf Wendel <ulf.wendel@phpdoc.de>
+ *         Pierre-Alain Joye <pajoye@php.net>
+ *         David Soria Parra <dsp@php.net>
+ *
+ * @category HTML
+ * @package  HTML_Template_IT
+ * @author   Ulf Wendel <uw@netuse.de>
+ * @license  BSD http://www.opensource.org/licenses/bsd-license.php
+ * @version  CVS: $Id$
+ * @link     http://pear.php.net/packages/HTML_Template_IT
+ * @access   public
+ */
 
 require_once 'PEAR.php';
 
-define('IT_OK',                         1);
-define('IT_ERROR',                     -1);
-define('IT_TPL_NOT_FOUND',             -2);
-define('IT_BLOCK_NOT_FOUND',           -3);
-define('IT_BLOCK_DUPLICATE',           -4);
-define('IT_UNKNOWN_OPTION',            -6);
+define('IT_OK', 1);
+define('IT_ERROR', -1);
+define('IT_TPL_NOT_FOUND', -2);
+define('IT_BLOCK_NOT_FOUND', -3);
+define('IT_BLOCK_DUPLICATE', -4);
+define('IT_UNKNOWN_OPTION', -6);
+
 /**
  * Integrated Template - IT
  *
@@ -42,8 +53,8 @@ define('IT_UNKNOWN_OPTION',            -6);
  *
  * I noticed that I do not any control on which block gets parsed into which one.
  * If all blocks are within one file, the script knows how they are nested and in
- * which way you have to parse them. IT knows that inner1 is a child of block2, there's
- * no need to tell him about this.
+ * which way you have to parse them. IT knows that inner1 is a child of block2,
+ * there's no need to tell him about this.
  *
  * <table border>
  *   <tr>
@@ -90,7 +101,8 @@ define('IT_UNKNOWN_OPTION',            -6);
  * </code>
  *
  * This will result in one repition of block1 which contains two repitions
- * of inner1. inner2 will be removed if $removeEmptyBlock is set to true which is the default.
+ * of inner1. inner2 will be removed if $removeEmptyBlock is set to true
+ * which is the default.
  *
  * Usage:
  * <code>
@@ -116,10 +128,12 @@ define('IT_UNKNOWN_OPTION',            -6);
  * $tpl->get();
  * </code>
  *
- * @author   Ulf Wendel <uw@netuse.de>
- * @version  $Id: IT.php,v 1.14 2005/11/01 10:14:19 pajoye Exp $
- * @access   public
+ * @category HTML
  * @package  HTML_Template_IT
+ * @author   Ulf Wendel <uw@netuse.de>
+ * @license  BSD http://www.opensource.org/licenses/bsd-license.php
+ * @link     http://pear.php.net/packages/HTML_Template_IT
+ * @access   public
  */
 class HTML_Template_IT
 {
@@ -133,7 +147,9 @@ class HTML_Template_IT
 
     /**
      * Clear cache on get()?
+
      * @var      boolean
+     * @acces    public
      */
     var $clearCache = false;
 
@@ -151,7 +167,7 @@ class HTML_Template_IT
      * @access   public
      * @see      $openingDelimiter, $blocknameRegExp, $variablenameRegExp
      */
-    var $closingDelimiter     = '}';
+    var $closingDelimiter = '}';
 
     /**
      * RegExp matching a block in the template.
@@ -161,7 +177,7 @@ class HTML_Template_IT
      * @access   public
      * @see      $variablenameRegExp, $openingDelimiter, $closingDelimiter
      */
-    var $blocknameRegExp    = '[0-9A-Za-z_-]+';
+    var $blocknameRegExp = '[\.0-9A-Za-z_-]+';
 
     /**
      * RegExp matching a variable placeholder in the template.
@@ -171,18 +187,19 @@ class HTML_Template_IT
      * @access   public
      * @see      $blocknameRegExp, $openingDelimiter, $closingDelimiter
      */
-    var $variablenameRegExp    = '[0-9A-Za-z_-]+';
+    var $variablenameRegExp = '[\.0-9A-Za-z_-]+';
 
     /**
      * RegExp used to find variable placeholder, filled by the constructor.
      * @var      string    Looks somewhat like @(delimiter varname delimiter)@
-     * @access   public
+     * @access   private
      * @see      IntegratedTemplate()
      */
     var $variablesRegExp = '';
 
     /**
      * RegExp used to strip unused variable placeholder.
+     * @access  private
      * @brother  $variablesRegExp
      */
     var $removeVariablesRegExp = '';
@@ -205,18 +222,21 @@ class HTML_Template_IT
      * RegExp used to find blocks an their content, filled by the constructor.
      * @var      string
      * @see      IntegratedTemplate()
+     * @access   private
      */
     var $blockRegExp = '';
 
     /**
      * Name of the current block.
      * @var      string
+     * @access   private
      */
     var $currentBlock = '__global__';
 
     /**
      * Content of the template.
      * @var      string
+     * @access   private
      */
     var $template = '';
 
@@ -225,6 +245,7 @@ class HTML_Template_IT
      *
      * @var      array
      * @see      findBlocks()
+     * @access   private
      */
     var $blocklist = array();
 
@@ -232,18 +253,21 @@ class HTML_Template_IT
      * Array with the parsed content of a block.
      *
      * @var      array
+     * @access   private
      */
     var $blockdata = array();
 
     /**
      * Array of variables in a block.
      * @var      array
+     * @access   private
      */
     var $blockvariables = array();
 
     /**
      * Array of inner blocks of a block.
      * @var      array
+     * @access   private
      */
     var $blockinner = array();
 
@@ -271,6 +295,7 @@ class HTML_Template_IT
      *
      * @var  array    $touchedBlocks
      * @see  touchBlock(), $removeEmptyBlocks
+     * @access private
      */
      var $touchedBlocks = array();
 
@@ -278,6 +303,7 @@ class HTML_Template_IT
      * List of blocks which should not be shown even if not "empty"
      * @var  array    $_hiddenBlocks
      * @see  hideBlock(), $removeEmptyBlocks
+     * @access private
      */
     var $_hiddenBlocks = array();
 
@@ -290,6 +316,7 @@ class HTML_Template_IT
      *
      * @var    array
      * @see    setVariable(), $clearCacheOnParse
+     * @access private
      */
     var $variableCache = array();
 
@@ -301,6 +328,7 @@ class HTML_Template_IT
      * add lots of values for unknown placeholder.
      *
      * @var    boolean
+     * @access public
      */
     var $clearCacheOnParse = false;
 
@@ -309,18 +337,21 @@ class HTML_Template_IT
      * The string gets prefixed to all filenames given.
      * @var    string
      * @see    HTML_Template_IT(), setRoot()
+     * @access private
      */
     var $fileRoot = '';
 
     /**
      * Internal flag indicating that a blockname was used multiple times.
      * @var    boolean
+     * @access private
      */
     var $flagBlocktrouble = false;
 
     /**
      * Flag indicating that the global block was parsed.
      * @var    boolean
+     * @access private
      */
     var $flagGlobalParsed = false;
 
@@ -335,11 +366,13 @@ class HTML_Template_IT
      * Now IT could notice this and skip the preparse.
      *
      * @var    boolean
+     * @access private
      */
     var $flagCacheTemplatefile = true;
 
     /**
      * EXPERIMENTAL! FIXME!
+     * @access private
      */
     var $lastTemplatefile = '';
 
@@ -350,10 +383,14 @@ class HTML_Template_IT
      * $_options['use_preg'] Whether to use preg_replace instead of
      * str_replace in parse()
      * (this is a backwards compatibility feature, see also bugs #21951, #20392)
+     *
+     * @var    array
+     * @access private
      */
     var $_options = array(
         'preserve_data' => false,
-        'use_preg'      => true
+        'use_preg'      => true,
+        'preserve_input'=> true
     );
 
     /**
@@ -363,18 +400,23 @@ class HTML_Template_IT
      * Make sure that you call this constructor if you derive your template
      * class from this one.
      *
-     * @param    string    File root directory, prefix for all filenames
-     *                     given to the object.
+     * @param string $root    File root directory, prefix for all filenames
+     *                        given to the object.
+     * @param mixed  $options Unknown
+     *
      * @see      setRoot()
+     * @access   public
      */
-    function HTML_Template_IT($root = '', $options = null)
+     function __construct($root = '', $options = null)
     {
         if (!is_null($options)) {
             $this->setOptions($options);
         }
+
         $this->variablesRegExp = '@' . $this->openingDelimiter .
                                  '(' . $this->variablenameRegExp . ')' .
                                  $this->closingDelimiter . '@sm';
+
         $this->removeVariablesRegExp = '@' . $this->openingDelimiter .
                                        "\s*(" . $this->variablenameRegExp .
                                        ")\s*" . $this->closingDelimiter .'@sm';
@@ -389,33 +431,40 @@ class HTML_Template_IT
     /**
      * Sets the option for the template class
      *
+     * @param string $option option name
+     * @param mixed  $value  option value
+     *
      * @access public
-     * @param  string  option name
-     * @param  mixed   option value
      * @return mixed   IT_OK on success, error object on failure
      */
     function setOption($option, $value)
     {
+        switch ($option) {
+            case 'removeEmptyBlocks':
+                $this->removeEmptyBlocks = $value;
+                return IT_OK;
+        }
+
         if (array_key_exists($option, $this->_options)) {
             $this->_options[$option] = $value;
             return IT_OK;
         }
 
         return PEAR::raiseError(
-                $this->errorMessage(IT_UNKNOWN_OPTION) . ": '{$option}'",
-                IT_UNKNOWN_OPTION
-            );
+            $this->errorMessage(IT_UNKNOWN_OPTION) . ": '{$option}'",
+            IT_UNKNOWN_OPTION
+        );
     }
 
     /**
      * Sets the options for the template class
      *
+     * @param string[] $options options array of options
+     *                           default value:
+     *                           'preserve_data' => false,
+     *                           'use_preg'      => true
+     *
      * @access public
-     * @param  string  options array of options
-     *                 default value:
-     *                   'preserve_data' => false,
-     *                   'use_preg'      => true
-     * @param  mixed   option value
      * @return mixed   IT_OK on success, error object on failure
      * @see $options
      */
@@ -435,7 +484,12 @@ class HTML_Template_IT
 
     /**
      * Print a certain block with all replacements done.
+     *
+     * @param string $block block
+     *
      * @brother get()
+     * @access public
+     * @return null
      */
     function show($block = '__global__')
     {
@@ -445,7 +499,8 @@ class HTML_Template_IT
     /**
      * Returns a block with all replacements done.
      *
-     * @param    string     name of the block
+     * @param string $block name of the block
+     *
      * @return   string
      * @throws   PEAR_Error
      * @access   public
@@ -459,25 +514,29 @@ class HTML_Template_IT
 
         if (!isset($this->blocklist[$block])) {
             $this->err[] = PEAR::raiseError(
-                            $this->errorMessage(IT_BLOCK_NOT_FOUND) .
-                            '"' . $block . "'",
-                            IT_BLOCK_NOT_FOUND
-                        );
+                $this->errorMessage(IT_BLOCK_NOT_FOUND) . '"' . $block . "'",
+                IT_BLOCK_NOT_FOUND
+            );
             return '';
         }
 
         if (isset($this->blockdata[$block])) {
             $ret = $this->blockdata[$block];
+
             if ($this->clearCache) {
                 unset($this->blockdata[$block]);
+                if ($block == '__global__') {
+                    $this->flagGlobalParsed = false;
+                }
             }
+
             if ($this->_options['preserve_data']) {
                 $ret = str_replace(
-                        $this->openingDelimiter .
-                        '%preserved%' . $this->closingDelimiter,
-                        $this->openingDelimiter,
-                        $ret
-                    );
+                    $this->openingDelimiter .
+                    '%preserved%' . $this->closingDelimiter,
+                    $this->openingDelimiter,
+                    $ret
+                );
             }
             return $ret;
         }
@@ -488,10 +547,13 @@ class HTML_Template_IT
     /**
      * Parses the given block.
      *
-     * @param    string    name of the block to be parsed
+     * @param string $block          name of the block to be parsed
+     * @param bool   $flag_recursion unknown
+     *
      * @access   public
      * @see      parseCurrentBlock()
      * @throws   PEAR_Error
+     * @return null
      */
     function parse($block = '__global__', $flag_recursion = false)
     {
@@ -499,9 +561,9 @@ class HTML_Template_IT
 
         if (!isset($this->blocklist[$block])) {
             return PEAR::raiseError(
-                $this->errorMessage( IT_BLOCK_NOT_FOUND ) . '"' . $block . "'",
-                        IT_BLOCK_NOT_FOUND
-                );
+                $this->errorMessage(IT_BLOCK_NOT_FOUND) . '"' . $block . "'",
+                IT_BLOCK_NOT_FOUND
+            );
         }
 
         if ($block == '__global__') {
@@ -515,23 +577,32 @@ class HTML_Template_IT
         $outer = $this->blocklist[$block];
         $empty = true;
 
+        $variablelist = array();
         if ($this->clearCacheOnParse) {
             foreach ($this->variableCache as $name => $value) {
                 $regs[] = $this->openingDelimiter .
                           $name . $this->closingDelimiter;
+
                 $values[] = $value;
+
                 $empty = false;
+
+                $variablelist[] = $name;
             }
             $this->variableCache = array();
         } else {
             foreach ($this->blockvariables[$block] as $allowedvar => $v) {
 
                 if (isset($this->variableCache[$allowedvar])) {
-                   $regs[]   = $this->openingDelimiter .
+                    $regs[]   = $this->openingDelimiter .
                                $allowedvar . $this->closingDelimiter;
-                   $values[] = $this->variableCache[$allowedvar];
-                   unset($this->variableCache[$allowedvar]);
-                   $empty = false;
+                    $values[] = $this->variableCache[$allowedvar];
+
+                    unset($this->variableCache[$allowedvar]);
+
+                    $empty = false;
+
+                    $variablelist[] = $allowedvar;
                 }
             }
         }
@@ -546,10 +617,12 @@ class HTML_Template_IT
 
                 $placeholder = $this->openingDelimiter . "__" .
                                 $innerblock . "__" . $this->closingDelimiter;
+
                 $outer = str_replace(
-                                    $placeholder,
-                                    $this->blockdata[$innerblock], $outer
-                        );
+                    $placeholder,
+                    $this->blockdata[$innerblock], $outer
+                );
+
                 $this->blockdata[$innerblock] = "";
             }
 
@@ -557,10 +630,9 @@ class HTML_Template_IT
 
         if (!$flag_recursion && 0 != count($values)) {
             if ($this->_options['use_preg']) {
-                $regs        = array_map(array(
-                                    &$this, '_addPregDelimiters'),
-                                    $regs
-                                );
+                $regs   = array_map(array(&$this, '_addPregDelimiters'), $regs);
+                $values = array_map(array(&$this, '_escapeBackreferences'), $values);
+
                 $funcReplace = 'preg_replace';
             } else {
                 $funcReplace = 'str_replace';
@@ -568,20 +640,25 @@ class HTML_Template_IT
 
             if ($this->_options['preserve_data']) {
                 $values = array_map(
-                            array(&$this, '_preserveOpeningDelimiter'), $values
-                        );
+                    array(&$this, '_preserveOpeningDelimiter'),
+                    $values
+                );
             }
 
             $outer = $funcReplace($regs, $values, $outer);
+        }
 
-            if ($this->removeUnknownVariables) {
-                $outer = preg_replace($this->removeVariablesRegExp, "", $outer);
-            }
+        if ($this->removeUnknownVariables) {
+            $outer = $this->removeUnknownVariablesFromBlock(
+                $block,
+                $outer,
+                $variablelist
+            );
         }
 
         if ($empty) {
             if (!$this->removeEmptyBlocks) {
-                $this->blockdata[$block ].= $outer;
+                $this->blockdata[$block ] .= $outer;
             } else {
                 if (isset($this->touchedBlocks[$block])) {
                     $this->blockdata[$block] .= $outer;
@@ -589,22 +666,85 @@ class HTML_Template_IT
                 }
             }
         } else {
-            $this->blockdata[$block] .= $outer;
+            if (empty($this->blockdata[$block])) {
+                $this->blockdata[$block] = $outer;
+            } else {
+                $this->blockdata[$block] .= $outer;
+            }
         }
 
         return $empty;
     } // end func parse
 
+    /**
+     * Removes unknown variables from block. If preserve_input is set to true
+     * only unknown variables that were present during setTemplate or
+     * loadTemplatefile are removed. Thus you can set a variable to
+     * "{SOMEINPUTDATA}" which is preserved.
+     *
+     * @param string $blockname    block
+     * @param string $blockinner   unknown
+     * @param string $variableList unknown
+     *
+     * @see parse()
+     * @access private
+     * @return null
+     */
+    function removeUnknownVariablesFromBlock ($blockname, $blockinner, $variableList)
+    {
+        if ($this->_options['preserve_input']) {
+            foreach ($this->blockvariables[$blockname] as $var => $setted) {
+                if (!in_array($var, $variableList)) {
+                    $blockinner = str_replace(
+                        $this->openingDelimiter . $var . $this->closingDelimiter,
+                        '',
+                        $blockinner
+                    );
+                }
+            }
+        } else {
+            $blockinner = preg_replace(
+                $this->removeVariablesRegExp,
+                '',
+                $blockinner
+            );
+        }
+
+        return $blockinner;
+    }
+
     /**
      * Parses the current block
+     *
      * @see      parse(), setCurrentBlock(), $currentBlock
      * @access   public
+     * @return null
      */
     function parseCurrentBlock()
     {
         return $this->parse($this->currentBlock);
     } // end func parseCurrentBlock
 
+    /**
+     * Checks to see if a placeholder exists within a block (and its children)
+     *
+     * @access public
+     * @return bool
+     */
+    function checkPlaceholderExists($blockname, $placeholder) {
+        if (isset($this->blockvariables[$blockname][$placeholder])) {
+            return true;
+        }
+        if (isset($this->blockinner[$blockname])) {
+            foreach ($this->blockinner[$blockname] as $block) {
+                if ($this->checkPlaceholderExists($block, $placeholder)) {
+                    return true;
+                }
+            }
+        }
+        return false;
+    } // end func checkPlaceholderExists
+
     /**
      * Sets a variable value.
      *
@@ -612,21 +752,24 @@ class HTML_Template_IT
      * or with one array $variables["varname"] = "value"
      * given setVariable($variables) quite like phplib templates set_var().
      *
-     * @param    mixed     string with the variable name or an array
-     *                     %variables["varname"] = "value"
-     * @param    string    value of the variable or empty if $variable
-     *                     is an array.
-     * @param    string    prefix for variable names
-     * @access   public
+     * @param mixed  $variable string with the variable name or an array
+     *                         %variables["varname"] = "value"
+     * @param string $value    value of the variable or empty if $variable
+     *                         is an array.
+     *
+     * @access public
+     * @return null
      */
     function setVariable($variable, $value = '')
     {
         if (is_array($variable)) {
-            $this->variableCache = array_merge(
-                                            $this->variableCache, $variable
-                                    );
+            foreach ($variable as $key => $value) {
+                $this->setVariable($key, $value);
+            }
         } else {
-            $this->variableCache[$variable] = $value;
+            if ($this->checkPlaceholderExists($this->currentBlock, $variable)) {
+                $this->variableCache[$variable] = $value;
+            }
         }
     } // end func setVariable
 
@@ -634,7 +777,8 @@ class HTML_Template_IT
      * Sets the name of the current block that is the block where variables
      * are added.
      *
-     * @param    string      name of the block
+     * @param string $block name of the block
+     *
      * @return   boolean     false on failure, otherwise true
      * @throws   PEAR_Error
      * @access   public
@@ -644,8 +788,9 @@ class HTML_Template_IT
 
         if (!isset($this->blocklist[$block])) {
             return PEAR::raiseError(
-                $this->errorMessage( IT_BLOCK_NOT_FOUND ) .
-                '"' . $block . "'", IT_BLOCK_NOT_FOUND
+                $this->errorMessage(IT_BLOCK_NOT_FOUND)
+                . '"' . $block . "'",
+                IT_BLOCK_NOT_FOUND
             );
         }
 
@@ -657,7 +802,8 @@ class HTML_Template_IT
     /**
      * Preserves an empty block even if removeEmptyBlocks is true.
      *
-     * @param    string      name of the block
+     * @param string $block name of the block
+     *
      * @return   boolean     false on false, otherwise true
      * @throws   PEAR_Error
      * @access   public
@@ -667,8 +813,9 @@ class HTML_Template_IT
     {
         if (!isset($this->blocklist[$block])) {
             return PEAR::raiseError(
-                $this->errorMessage(IT_BLOCK_NOT_FOUND) .
-                '"' . $block . "'", IT_BLOCK_NOT_FOUND);
+                $this->errorMessage(IT_BLOCK_NOT_FOUND) . '"' . $block . "'",
+                IT_BLOCK_NOT_FOUND
+            );
         }
 
         $this->touchedBlocks[$block] = true;
@@ -683,8 +830,9 @@ class HTML_Template_IT
      * when a new template is given. Don't use this function
      * unless you know what you're doing.
      *
-     * @access   public
+     * @access   private
      * @see      free()
+     * @return null
      */
     function init()
     {
@@ -700,8 +848,9 @@ class HTML_Template_IT
      *
      * Don't use this function unless you know what you're doing.
      *
-     * @access   public
+     * @access   private
      * @see      init()
+     * @return null
      */
     function free()
     {
@@ -709,9 +858,9 @@ class HTML_Template_IT
 
         $this->currentBlock = '__global__';
 
-        $this->variableCache    = array();
-        $this->blocklookup      = array();
-        $this->touchedBlocks    = array();
+        $this->variableCache = array();
+        $this->blocklist     = array();
+        $this->touchedBlocks = array();
 
         $this->flagBlocktrouble = false;
         $this->flagGlobalParsed = false;
@@ -723,23 +872,26 @@ class HTML_Template_IT
      * You can eighter load a template file from disk with
      * LoadTemplatefile() or set the template manually using this function.
      *
-     * @param        string      template content
-     * @param        boolean     remove unknown/unused variables?
-     * @param        boolean     remove empty blocks?
+     * @param string $template               template content
+     * @param bool   $removeUnknownVariables how to handle unknown variables.
+     * @param bool   $removeEmptyBlocks      how to handle empty blocks.
+     *
      * @see          LoadTemplatefile(), $template
      * @access       public
+     * @return       boolean
      */
-    function setTemplate( $template, $removeUnknownVariables = true,
-                          $removeEmptyBlocks = true)
-    {
+    function setTemplate( $template,
+        $removeUnknownVariables = true,
+    $removeEmptyBlocks = true) {
         $this->removeUnknownVariables = $removeUnknownVariables;
+
         $this->removeEmptyBlocks = $removeEmptyBlocks;
 
         if ($template == '' && $this->flagCacheTemplatefile) {
             $this->variableCache = array();
-            $this->blockdata = array();
+            $this->blockdata     = array();
             $this->touchedBlocks = array();
-            $this->currentBlock = '__global__';
+            $this->currentBlock  = '__global__';
         } else {
             $this->template = '<!-- BEGIN __global__ -->' . $template .
                               '<!-- END __global__ -->';
@@ -756,21 +908,21 @@ class HTML_Template_IT
     /**
      * Reads a template file from the disk.
      *
-     * @param    string      name of the template file
-     * @param    bool        how to handle unknown variables.
-     * @param    bool        how to handle empty blocks.
+     * @param string $filename               name of the template file
+     * @param bool   $removeUnknownVariables how to handle unknown variables.
+     * @param bool   $removeEmptyBlocks      how to handle empty blocks.
+     *
      * @access   public
      * @return   boolean    false on failure, otherwise true
      * @see      $template, setTemplate(), $removeUnknownVariables,
      *           $removeEmptyBlocks
      */
     function loadTemplatefile( $filename,
-                               $removeUnknownVariables = true,
-                               $removeEmptyBlocks = true )
-    {
+        $removeUnknownVariables = true,
+    $removeEmptyBlocks = true ) {;
         $template = '';
-        if (!$this->flagCacheTemplatefile ||
-            $this->lastTemplatefile != $filename
+        if (!$this->flagCacheTemplatefile
+            || $this->lastTemplatefile != $filename
         ) {
             $template = $this->getFile($filename);
         }
@@ -778,8 +930,9 @@ class HTML_Template_IT
 
         return $template != '' ?
                 $this->setTemplate(
-                        $template,$removeUnknownVariables, $removeEmptyBlocks
-                    ) : false;
+                    $template,
+                    $removeUnknownVariables,
+                $removeEmptyBlocks) : false;
     } // end func LoadTemplatefile
 
     /**
@@ -789,9 +942,11 @@ class HTML_Template_IT
      * Make sure that you override this function when using the class
      * on windows.
      *
-     * @param    string
-     * @see      IntegratedTemplate()
+     * @param string $root File root
+     *
+     * @see      HTML_Template_IT()
      * @access   public
+     * @return null
      */
     function setRoot($root)
     {
@@ -804,6 +959,9 @@ class HTML_Template_IT
 
     /**
      * Build a list of all variables within of a block
+     *
+     * @access private
+     * @return null
      */
     function buildBlockvariablelist()
     {
@@ -822,6 +980,9 @@ class HTML_Template_IT
 
     /**
      * Returns a list of all global variables
+     *
+     * @access public
+     * @return array
      */
     function getGlobalvariables()
     {
@@ -843,8 +1004,11 @@ class HTML_Template_IT
     /**
      * Recusively builds a list of all blocks within the template.
      *
-     * @param    string    string that gets scanned
+     * @param string $string string that gets scanned
+     *
+     * @access   private
      * @see      $blocklist
+     * @return   array
      */
     function findBlocks($string)
     {
@@ -852,15 +1016,14 @@ class HTML_Template_IT
 
         if (preg_match_all($this->blockRegExp, $string, $regs, PREG_SET_ORDER)) {
             foreach ($regs as $k => $match) {
-                $blockname         = $match[1];
+                $blockname    = $match[1];
                 $blockcontent = $match[2];
 
                 if (isset($this->blocklist[$blockname])) {
-                    $this->err[] = PEAR::raiseError(
-                                            $this->errorMessage(
-                                            IT_BLOCK_DUPLICATE, $blockname),
-                                            IT_BLOCK_DUPLICATE
-                                    );
+                    $msg = $this->errorMessage(IT_BLOCK_DUPLICATE, $blockname);
+
+                    $this->err[] = PEAR::raiseError($msg, IT_BLOCK_DUPLICATE);
+
                     $this->flagBlocktrouble = true;
                 }
 
@@ -870,21 +1033,20 @@ class HTML_Template_IT
                 $blocklist[] = $blockname;
 
                 $inner = $this->findBlocks($blockcontent);
+                $regex = '@<!--\s+BEGIN\s+%s\s+-->(.*)<!--\s+END\s+%s\s+-->@sm';
                 foreach ($inner as $k => $name) {
-                    $pattern = sprintf(
-                        '@<!--\s+BEGIN\s+%s\s+-->(.*)<!--\s+END\s+%s\s+-->@sm',
-                        $name,
-                        $name
-                    );
+                    $pattern = sprintf($regex, preg_quote($name), preg_quote($name));
 
                     $this->blocklist[$blockname] = preg_replace(
-                                        $pattern,
-                                        $this->openingDelimiter .
-                                        '__' . $name . '__' .
-                                        $this->closingDelimiter,
-                                        $this->blocklist[$blockname]
-                               );
+                        $pattern,
+                        $this->openingDelimiter .
+                        '__' . $name . '__' .
+                        $this->closingDelimiter,
+                        $this->blocklist[$blockname]
+                    );
+
                     $this->blockinner[$blockname][] = $name;
+
                     $this->blockparents[$name] = $blockname;
                 }
             }
@@ -895,8 +1057,11 @@ class HTML_Template_IT
 
     /**
      * Reads a file from disk and returns its content.
-     * @param    string    Filename
+     *
+     * @param string $filename Filename
+     *
      * @return   string    Filecontent
+     * @access   private
      */
     function getFile($filename)
     {
@@ -908,18 +1073,25 @@ class HTML_Template_IT
 
         if (!($fh = @fopen($filename, 'r'))) {
             $this->err[] = PEAR::raiseError(
-                        $this->errorMessage(IT_TPL_NOT_FOUND) .
-                        ': "' .$filename .'"',
-                        IT_TPL_NOT_FOUND
-                    );
+                $this->errorMessage(IT_TPL_NOT_FOUND) . ': "' .$filename .'"',
+                IT_TPL_NOT_FOUND
+            );
             return "";
         }
 
-        $content = fread($fh, filesize($filename));
+        $fsize = filesize($filename);
+        if ($fsize < 1) {
+            fclose($fh);
+            return '';
+        }
+
+        $content = fread($fh, $fsize);
         fclose($fh);
 
-        return preg_replace(
-            "#<!-- INCLUDE (.*) -->#ime", "\$this->getFile('\\1')", $content
+        return preg_replace_callback(
+            "#<!-- INCLUDE (.*) -->#im",
+            function ($m) { return $this->getFile($m[1]); },
+            $content
         );
     } // end func getFile
 
@@ -927,20 +1099,42 @@ class HTML_Template_IT
      * Adds delimiters to a string, so it can be used as a pattern
      * in preg_* functions
      *
-     * @param string
+     * @param string $str input
+     *
      * @return string
+     * @access private
      */
     function _addPregDelimiters($str)
     {
-        return '@' . $str . '@';
+        return '@' . preg_quote($str) . '@';
     }
 
-   /**
-    * Replaces an opening delimiter by a special string
-    *
-    * @param string
-    * @return string
-    */
+    /**
+     * Escapes $ and \ as preg_replace will treat
+     * them as a backreference and not literal.
+     * See bug #9501
+     *
+     * @param string $str String to escape
+     *
+     * @since 1.2.2
+     * @return string
+     * @access private
+     */
+    function _escapeBackreferences($str)
+    {
+        $str = str_replace('\\', '\\\\', $str);
+        $str = preg_replace('@\$([0-9]{1,2})@', '\\\$${1}', $str);
+        return $str;
+    }
+
+    /**
+     * Replaces an opening delimiter by a special string
+     *
+     * @param string $str special string
+     *
+     * @return string
+     * @access private
+     */
     function _preserveOpeningDelimiter($str)
     {
         return (false === strpos($str, $this->openingDelimiter))?
@@ -956,8 +1150,10 @@ class HTML_Template_IT
     /**
      * Return a textual error message for a IT error code
      *
-     * @param integer $value error code
+     * @param integer $value     error code
+     * @param string  $blockname unknown
      *
+     * @access private
      * @return string error message, or false if the error code was
      * not recognized
      */
